import os

from PySide6.QtCore import QObject, QProcess, Signal, QTimer


class ConsoleProcessAdapter(QObject):
    started = Signal()
    finished = Signal(int, QProcess.ExitStatus)
    stdout = Signal(str)
    stderr = Signal(str)
    system = Signal(str)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.proc = QProcess(self)
        self.proc.setProcessChannelMode(QProcess.ProcessChannelMode.SeparateChannels)
        self.proc.readyReadStandardOutput.connect(self._on_stdout)
        self.proc.readyReadStandardError.connect(self._on_stderr)
        self.proc.started.connect(self.started)
        self.proc.finished.connect(self.finished)

    def set_cwd(self, path: str):
        self.proc.setWorkingDirectory(path)

    def start_shell(self, shell: str = "auto"):
        if self.is_running():
            self.system.emit("Shell already running")
            return
        program, args = self._build_shell_command(shell)
        self.proc.start(program, args)

        if os.name == "nt":
            QTimer.singleShot(500, lambda: self.send_command("chcp 65001"))

    def send_command(self, cmd: str):
        if not self.is_running():
            self.system.emit("Shell not running")
            return
        self.proc.write((cmd + "\n").encode("utf-8"))

    def stop(self):
        if not self.is_running():
            return
        self.proc.terminate()
        QTimer.singleShot(1000, lambda: self.is_running() and self.proc.kill())

    def is_running(self):
        return self.proc.state() != QProcess.ProcessState.NotRunning

    def _on_stdout(self):
        self.stdout.emit(bytes(self.proc.readAllStandardOutput()).decode("utf-8", errors="replace"))

    def _on_stderr(self):
        self.stderr.emit(bytes(self.proc.readAllStandardError()).decode("utf-8", errors="replace"))

    def _build_shell_command(self, shell: str):
        if shell == "auto":
            shell = "cmd" if os.name == "nt" else "bash"
        if os.name == "nt":
            if shell == "powershell":
                return "powershell.exe", ["-NoLogo", "-NoProfile", "-ExecutionPolicy", "Bypass"]
            return "cmd.exe", []
        return ("/bin/bash" if shell == "bash" else "/bin/sh"), []



from queue import Queue

from watchdog.events import FileSystemEventHandler

from je_editor.code_scan.ruff_thread import RuffThread


class RuffPythonFileChangeHandler(FileSystemEventHandler):

    def __init__(self, ruff_commands: list = None):
        super(RuffPythonFileChangeHandler, self).__init__()
        self.ruff_commands = ruff_commands
        self.stdout_queue = Queue()
        self.stderr_queue = Queue()
        self.ruff_threads_dict = dict()

    def on_modified(self, event):
        if event.is_directory:
            return
        if event.src_path.endswith(".py"):
            if self.ruff_threads_dict.get(event.src_path) is None:
                ruff_thread = RuffThread(self.ruff_commands, self.stdout_queue, self.stderr_queue)
                self.ruff_threads_dict.update({event.src_path: ruff_thread})
                ruff_thread.start()
            else:
                ruff_thread = self.ruff_threads_dict.get(event.src_path)
                if not ruff_thread.is_alive():
                    ruff_thread = RuffThread(self.ruff_commands, self.stdout_queue, self.stderr_queue)
                    self.ruff_threads_dict.update({event.src_path: ruff_thread})
                    ruff_thread.start()
                else:
                    pass





import subprocess
import threading
import time
from queue import Queue


class RuffThread(threading.Thread):

    def __init__(self, ruff_commands: list, std_queue: Queue, stderr_queue: Queue):
        super().__init__()
        if ruff_commands is None:
            self.ruff_commands = ["ruff", "check"]
        else:
            self.ruff_commands = ruff_commands
        self.ruff_process = None
        self.std_queue = std_queue
        self.stderr_queue = stderr_queue

    def run(self):
        self.ruff_process = subprocess.Popen(
            self.ruff_commands, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, bufsize=1)
        while self.ruff_process.poll() is None:
            time.sleep(1)
        else:
            for line in self.ruff_process.stdout:
                print(line.strip())
            for line in self.ruff_process.stderr:
                print(line.strip())






import threading
import time

from watchdog.observers import Observer

from je_editor.code_scan.watchdog_implement import RuffPythonFileChangeHandler


class WatchdogThread(threading.Thread):

    def __init__(self, check_path: str):
        super().__init__()
        self.check_path = check_path
        self.ruff_handler = RuffPythonFileChangeHandler()
        self.running = True

    def run(self):
        observer = Observer()
        observer.schedule(self.ruff_handler, str(self.check_path), recursive=True)
        observer.start()
        try:
            while self.running:
                time.sleep(1)
        finally:
            observer.stop()


if __name__ == '__main__':
    watchdog_thread = WatchdogThread(".")
    watchdog_thread.start()
    while True:
        time.sleep(1)




import ast

from PySide6.QtCore import QAbstractTableModel, Qt, QTimer, QSortFilterProxyModel
from PySide6.QtWidgets import QTableView, QVBoxLayout, QWidget, QLineEdit, QLabel

from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


class VariableModel(QAbstractTableModel):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.variables = []

    def update_data(self):
        parent_widget = self.parent()
        if parent_widget and getattr(parent_widget, "table", None):
            if parent_widget.table.state() != QTableView.State.NoState:
                return

        vars_dict = globals()
        self.beginResetModel()
        self.variables = [
            [name, type(value).__name__, repr(value), value]
            for name, value in vars_dict.items()
            if not name.startswith("__")  # 過濾內建變數
        ]
        self.endResetModel()

    def rowCount(self, parent=None):
        return len(self.variables)

    def columnCount(self, parent=None):
        return 3

    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        if not index.isValid() or not (0 <= index.row() < len(self.variables)):
            return None
        if role == Qt.ItemDataRole.DisplayRole:
            return self.variables[index.row()][index.column()]
        return None

    def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal:
            return [
                language_wrapper.language_word_dict.get("variable_inspector_var_name"),
                language_wrapper.language_word_dict.get("variable_inspector_var_type"),
                language_wrapper.language_word_dict.get("variable_inspector_var_value")
            ][section]
        return None

    def flags(self, index):
        if not index.isValid() or not (0 <= index.row() < len(self.variables)):
            return Qt.ItemFlag.NoItemFlags
        base = super().flags(index)
        if index.column() == 2:  # 只允許編輯值欄
            base |= Qt.ItemFlag.ItemIsEditable
        return base

    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
        if role == Qt.ItemDataRole.EditRole and index.column() == 2:

            if value == "":
                return False

            old_repr = self.variables[index.row()][2]
            if str(value) == old_repr:
                return False

            var_name = self.variables[index.row()][0]
            try:
                new_value = ast.literal_eval(value)
            except Exception:
                new_value = value

            if new_value == self.variables[index.row()][3]:
                return False

            globals()[var_name] = new_value
            self.variables[index.row()][2] = repr(new_value)
            self.variables[index.row()][3] = new_value
            self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole])
            return True
        return False

class VariableProxy(QSortFilterProxyModel):
    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
        return self.sourceModel().setData(
            self.mapToSource(index), value, role
        )

class VariableInspector(QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle(language_wrapper.language_word_dict.get("variable_inspector_title"))
        layout = QVBoxLayout(self)

        self.search_box = QLineEdit()
        self.search_box.setPlaceholderText(language_wrapper.language_word_dict.get("variable_inspector_search"))
        layout.addWidget(QLabel(language_wrapper.language_word_dict.get("variable_inspector_search")))
        layout.addWidget(self.search_box)

        self.model = VariableModel(parent=self)
        self.proxy_model = VariableProxy(self)
        self.proxy_model.setSourceModel(self.model)
        self.proxy_model.setFilterCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
        self.proxy_model.setFilterKeyColumn(0)

        self.table = QTableView()
        self.table.setModel(self.proxy_model)
        self.table.horizontalHeader().setStretchLastSection(True)
        self.table.horizontalHeader().setSectionResizeMode(
            self.table.horizontalHeader().ResizeMode.Stretch
        )
        layout.addWidget(self.table, stretch=1)

        self.search_box.textChanged.connect(
            self.proxy_model.setFilterFixedString
        )

        self.timer = QTimer(self)
        self.timer.timeout.connect(self.model.update_data)
        self.timer.start(500)
